#!/usr/bin/env python
# coding: utf-8

# Copyright (c) Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.

"""
This file originates from the 'jupyter-packaging' package, and
contains a set of useful utilities for including npm packages
within a Python package.
"""
from collections import defaultdict
from os.path import join as pjoin
import io
import os
import functools
import pipes
import re
import shlex
import subprocess
import sys


# BEFORE importing distutils, remove MANIFEST. distutils doesn't properly
# update it when the contents of directories change.
if os.path.exists("MANIFEST"):
    os.remove("MANIFEST")


from distutils.cmd import Command
from distutils.command.build_py import build_py
from distutils.command.sdist import sdist
from distutils import log

from setuptools.command.develop import develop
from setuptools.command.bdist_egg import bdist_egg

try:
    from wheel.bdist_wheel import bdist_wheel
except ImportError:
    bdist_wheel = None

if sys.platform == "win32":
    from subprocess import list2cmdline
else:

    def list2cmdline(cmd_list):
        return " ".join(map(pipes.quote, cmd_list))


__version__ = "0.2.0"

# ---------------------------------------------------------------------------
# Top Level Variables
# ---------------------------------------------------------------------------

HERE = os.path.abspath(os.path.dirname(__file__))
is_repo = os.path.exists(pjoin(HERE, ".git"))
node_modules = pjoin(HERE, "node_modules")

SEPARATORS = os.sep if os.altsep is None else os.sep + os.altsep

npm_path = ":".join(
    [pjoin(HERE, "node_modules", ".bin"), os.environ.get("PATH", os.defpath)]
)

if "--skip-npm" in sys.argv:
    print("Skipping npm install as requested.")
    skip_npm = True
    sys.argv.remove("--skip-npm")
else:
    skip_npm = False


# ---------------------------------------------------------------------------
# Public Functions
# ---------------------------------------------------------------------------


def get_version(file, name="__version__"):
    """Get the version of the package from the given file by
    executing it and extracting the given `name`.
    """
    path = os.path.realpath(file)
    version_ns = {}
    with io.open(path, encoding="utf8") as f:
        exec(f.read(), {}, version_ns)
    return version_ns[name]


def ensure_python(specs):
    """Given a list of range specifiers for python, ensure compatibility.
    """
    if not isinstance(specs, (list, tuple)):
        specs = [specs]
    v = sys.version_info
    part = "%s.%s" % (v.major, v.minor)
    for spec in specs:
        if part == spec:
            return
        try:
            if eval(part + spec):
                return
        except SyntaxError:
            pass
    raise ValueError("Python version %s unsupported" % part)


def find_packages(top=HERE):
    """
    Find all of the packages.
    """
    packages = []
    for d, dirs, _ in os.walk(top, followlinks=True):
        if os.path.exists(pjoin(d, "__init__.py")):
            packages.append(os.path.relpath(d, top).replace(os.path.sep, "."))
        elif d != top:
            # Do not look for packages in subfolders if current is not a package
            dirs[:] = []
    return packages


def update_package_data(distribution):
    """update build_py options to get package_data changes"""
    build_py = distribution.get_command_obj("build_py")
    build_py.finalize_options()


class bdist_egg_disabled(bdist_egg):
    """Disabled version of bdist_egg

    Prevents setup.py install performing setuptools' default easy_install,
    which it should never ever do.
    """

    def run(self):
        sys.exit(
            "Aborting implicit building of eggs. Use `pip install .` "
            " to install from source."
        )


def create_cmdclass(prerelease_cmd=None, package_data_spec=None, data_files_spec=None):
    """Create a command class with the given optional prerelease class.

    Parameters
    ----------
    prerelease_cmd: (name, Command) tuple, optional
        The command to run before releasing.
    package_data_spec: dict, optional
        A dictionary whose keys are the dotted package names and
        whose values are a list of glob patterns.
    data_files_spec: list, optional
        A list of (path, dname, pattern) tuples where the path is the
        `data_files` install path, dname is the source directory, and the
        pattern is a glob pattern.

    Notes
    -----
    We use specs so that we can find the files *after* the build
    command has run.

    The package data glob patterns should be relative paths from the package
    folder containing the __init__.py file, which is given as the package
    name.
    e.g. `dict(foo=['./bar/*', './baz/**'])`

    The data files directories should be absolute paths or relative paths
    from the root directory of the repository.  Data files are specified
    differently from `package_data` because we need a separate path entry
    for each nested folder in `data_files`, and this makes it easier to
    parse.
    e.g. `('share/foo/bar', 'pkgname/bizz, '*')`
    """
    wrapped = [prerelease_cmd] if prerelease_cmd else []
    if package_data_spec or data_files_spec:
        wrapped.append("handle_files")
    wrapper = functools.partial(_wrap_command, wrapped)
    handle_files = _get_file_handler(package_data_spec, data_files_spec)

    if "bdist_egg" in sys.argv:
        egg = wrapper(bdist_egg, strict=True)
    else:
        egg = bdist_egg_disabled

    cmdclass = dict(
        build_py=wrapper(build_py, strict=is_repo),
        bdist_egg=egg,
        sdist=wrapper(sdist, strict=True),
        handle_files=handle_files,
    )

    if bdist_wheel:
        cmdclass["bdist_wheel"] = wrapper(bdist_wheel, strict=True)

    cmdclass["develop"] = wrapper(develop, strict=True)
    return cmdclass


def command_for_func(func):
    """Create a command that calls the given function."""

    class FuncCommand(BaseCommand):
        def run(self):
            func()
            update_package_data(self.distribution)

    return FuncCommand


def run(cmd, **kwargs):
    """Echo a command before running it.  Defaults to repo as cwd"""
    log.info("> " + list2cmdline(cmd))
    kwargs.setdefault("cwd", HERE)
    kwargs.setdefault("shell", os.name == "nt")
    if not isinstance(cmd, (list, tuple)) and os.name != "nt":
        cmd = shlex.split(cmd)
    cmd_path = which(cmd[0])
    if not cmd_path:
        sys.exit(
            "Aborting. Could not find cmd (%s) in path. "
            "If command is not expected to be in user's path, "
            "use an absolute path." % cmd[0]
        )
    cmd[0] = cmd_path
    return subprocess.check_call(cmd, **kwargs)


def is_stale(target, source):
    """Test whether the target file/directory is stale based on the source
       file/directory.
    """
    if not os.path.exists(target):
        return True
    target_mtime = recursive_mtime(target) or 0
    return compare_recursive_mtime(source, cutoff=target_mtime)


class BaseCommand(Command):
    """Empty command because Command needs subclasses to override too much"""

    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def get_inputs(self):
        return []

    def get_outputs(self):
        return []


def combine_commands(*commands):
    """Return a Command that combines several commands."""

    class CombinedCommand(Command):
        user_options = []

        def initialize_options(self):
            self.commands = []
            for C in commands:
                self.commands.append(C(self.distribution))
            for c in self.commands:
                c.initialize_options()

        def finalize_options(self):
            for c in self.commands:
                c.finalize_options()

        def run(self):
            for c in self.commands:
                c.run()

    return CombinedCommand


def compare_recursive_mtime(path, cutoff, newest=True):
    """Compare the newest/oldest mtime for all files in a directory.

    Cutoff should be another mtime to be compared against. If an mtime that is
    newer/older than the cutoff is found it will return True.
    E.g. if newest=True, and a file in path is newer than the cutoff, it will
    return True.
    """
    if os.path.isfile(path):
        mt = mtime(path)
        if newest:
            if mt > cutoff:
                return True
        elif mt < cutoff:
            return True
    for dirname, _, filenames in os.walk(path, topdown=False):
        for filename in filenames:
            mt = mtime(pjoin(dirname, filename))
            if newest:  # Put outside of loop?
                if mt > cutoff:
                    return True
            elif mt < cutoff:
                return True
    return False


def recursive_mtime(path, newest=True):
    """Gets the newest/oldest mtime for all files in a directory."""
    if os.path.isfile(path):
        return mtime(path)
    current_extreme = None
    for dirname, dirnames, filenames in os.walk(path, topdown=False):
        for filename in filenames:
            mt = mtime(pjoin(dirname, filename))
            if newest:  # Put outside of loop?
                if mt >= (current_extreme or mt):
                    current_extreme = mt
            elif mt <= (current_extreme or mt):
                current_extreme = mt
    return current_extreme


def mtime(path):
    """shorthand for mtime"""
    return os.stat(path).st_mtime


def install_npm(
    path=None, build_dir=None, source_dir=None, build_cmd="build", force=False, npm=None
):
    """Return a Command for managing an npm installation.

    Note: The command is skipped if the `--skip-npm` flag is used.

    Parameters
    ----------
    path: str, optional
        The base path of the node package.  Defaults to the repo root.
    build_dir: str, optional
        The target build directory.  If this and source_dir are given,
        the JavaScript will only be build if necessary.
    source_dir: str, optional
        The source code directory.
    build_cmd: str, optional
        The npm command to build assets to the build_dir.
    npm: str or list, optional.
        The npm executable name, or a tuple of ['node', executable].
    """

    class NPM(BaseCommand):
        description = "install package.json dependencies using npm"

        def run(self):
            if skip_npm:
                log.info("Skipping npm-installation")
                return
            node_package = path or HERE
            node_modules = pjoin(node_package, "node_modules")
            is_yarn = os.path.exists(pjoin(node_package, "yarn.lock"))

            npm_cmd = npm

            if npm is None:
                if is_yarn:
                    npm_cmd = ["yarn"]
                else:
                    npm_cmd = ["npm"]

            if not which(npm_cmd[0]):
                log.error(
                    "`{0}` unavailable.  If you're running this command "
                    "using sudo, make sure `{0}` is availble to sudo".format(npm_cmd[0])
                )
                return

            if force or is_stale(node_modules, pjoin(node_package, "package.json")):
                log.info(
                    "Installing build dependencies with npm.  This may "
                    "take a while..."
                )
                run(npm_cmd + ["install"], cwd=node_package)
            if build_dir and source_dir and not force:
                should_build = is_stale(build_dir, source_dir)
            else:
                should_build = True
            if should_build:
                run(npm_cmd + ["run", build_cmd], cwd=node_package)

    return NPM


def ensure_targets(targets):
    """Return a Command that checks that certain files exist.

    Raises a ValueError if any of the files are missing.

    Note: The check is skipped if the `--skip-npm` flag is used.
    """

    class TargetsCheck(BaseCommand):
        def run(self):
            if skip_npm:
                log.info("Skipping target checks")
                return
            missing = [t for t in targets if not os.path.exists(t)]
            if missing:
                raise ValueError(("missing files: %s" % missing))

    return TargetsCheck


# `shutils.which` function copied verbatim from the Python-3.3 source.
def which(cmd, mode=os.F_OK | os.X_OK, path=None):
    """Given a command, mode, and a PATH string, return the path which
    conforms to the given mode on the PATH, or None if there is no such
    file.
    `mode` defaults to os.F_OK | os.X_OK. `path` defaults to the result
    of os.environ.get("PATH"), or can be overridden with a custom search
    path.
    """

    # Check that a given file can be accessed with the correct mode.
    # Additionally check that `file` is not a directory, as on Windows
    # directories pass the os.access check.
    def _access_check(fn, mode):
        return os.path.exists(fn) and os.access(fn, mode) and not os.path.isdir(fn)

    # Short circuit. If we're given a full path which matches the mode
    # and it exists, we're done here.
    if _access_check(cmd, mode):
        return cmd

    path = (path or os.environ.get("PATH", os.defpath)).split(os.pathsep)

    if sys.platform == "win32":
        # The current directory takes precedence on Windows.
        if os.curdir not in path:
            path.insert(0, os.curdir)

        # PATHEXT is necessary to check on Windows.
        pathext = os.environ.get("PATHEXT", "").split(os.pathsep)
        # See if the given file matches any of the expected path extensions.
        # This will allow us to short circuit when given "python.exe".
        matches = [cmd for ext in pathext if cmd.lower().endswith(ext.lower())]
        # If it does match, only test that one, otherwise we have to try
        # others.
        files = [cmd] if matches else [cmd + ext.lower() for ext in pathext]
    else:
        # On other platforms you don't have things like PATHEXT to tell you
        # what file suffixes are executable, so just pass on cmd as-is.
        files = [cmd]

    seen = set()
    for dir in path:
        dir = os.path.normcase(dir)
        if dir not in seen:
            seen.add(dir)
            for thefile in files:
                name = os.path.join(dir, thefile)
                if _access_check(name, mode):
                    return name
    return None


# ---------------------------------------------------------------------------
# Private Functions
# ---------------------------------------------------------------------------


def _wrap_command(cmds, cls, strict=True):
    """Wrap a setup command

    Parameters
    ----------
    cmds: list(str)
        The names of the other commands to run prior to the command.
    strict: boolean, optional
        Wether to raise errors when a pre-command fails.
    """

    class WrappedCommand(cls):
        def run(self):
            if not getattr(self, "uninstall", None):
                try:
                    [self.run_command(cmd) for cmd in cmds]
                except Exception:
                    if strict:
                        raise
                    else:
                        pass
            # update package data
            update_package_data(self.distribution)

            result = cls.run(self)
            return result

    return WrappedCommand


def _get_file_handler(package_data_spec, data_files_spec):
    """Get a package_data and data_files handler command.
    """

    class FileHandler(BaseCommand):
        def run(self):
            package_data = self.distribution.package_data
            package_spec = package_data_spec or dict()

            for (key, patterns) in package_spec.items():
                package_data[key] = _get_package_data(key, patterns)

            self.distribution.data_files = _get_data_files(
                data_files_spec, self.distribution.data_files
            )

    return FileHandler


def _glob_pjoin(*parts):
    """Join paths for glob processing"""
    if parts[0] in (".", ""):
        parts = parts[1:]
    return pjoin(*parts).replace(os.sep, "/")


def _get_data_files(data_specs, existing, top=HERE):
    """Expand data file specs into valid data files metadata.

    Parameters
    ----------
    data_specs: list of tuples
        See [create_cmdclass] for description.
    existing: list of tuples
        The existing distrubution data_files metadata.

    Returns
    -------
    A valid list of data_files items.
    """
    # Extract the existing data files into a staging object.
    file_data = defaultdict(list)
    for (path, files) in existing or []:
        file_data[path] = files

    # Extract the files and assign them to the proper data
    # files path.
    for (path, dname, pattern) in data_specs or []:
        if os.path.isabs(dname):
            dname = os.path.relpath(dname, top)
        dname = dname.replace(os.sep, "/")
        offset = 0 if dname in (".", "") else len(dname) + 1
        files = _get_files(_glob_pjoin(dname, pattern), top=top)
        for fname in files:
            # Normalize the path.
            root = os.path.dirname(fname)
            full_path = _glob_pjoin(path, root[offset:])
            print(dname, root, full_path, offset)
            if full_path.endswith("/"):
                full_path = full_path[:-1]
            file_data[full_path].append(fname)

    # Construct the data files spec.
    data_files = []
    for (path, files) in file_data.items():
        data_files.append((path, files))
    return data_files


def _get_files(file_patterns, top=HERE):
    """Expand file patterns to a list of paths.

    Parameters
    -----------
    file_patterns: list or str
        A list of glob patterns for the data file locations.
        The globs can be recursive if they include a `**`.
        They should be relative paths from the top directory or
        absolute paths.
    top: str
        the directory to consider for data files

    Note:
    Files in `node_modules` are ignored.
    """
    if not isinstance(file_patterns, (list, tuple)):
        file_patterns = [file_patterns]

    for i, p in enumerate(file_patterns):
        if os.path.isabs(p):
            file_patterns[i] = os.path.relpath(p, top)

    matchers = [_compile_pattern(p) for p in file_patterns]

    files = set()

    for root, dirnames, filenames in os.walk(top):
        # Don't recurse into node_modules
        if "node_modules" in dirnames:
            dirnames.remove("node_modules")
        for m in matchers:
            for filename in filenames:
                fn = os.path.relpath(_glob_pjoin(root, filename), top)
                fn = fn.replace(os.sep, "/")
                if m(fn):
                    files.add(fn.replace(os.sep, "/"))

    return list(files)


def _get_package_data(root, file_patterns=None):
    """Expand file patterns to a list of `package_data` paths.

    Parameters
    -----------
    root: str
        The relative path to the package root from `HERE`.
    file_patterns: list or str, optional
        A list of glob patterns for the data file locations.
        The globs can be recursive if they include a `**`.
        They should be relative paths from the root or
        absolute paths.  If not given, all files will be used.

    Note:
    Files in `node_modules` are ignored.
    """
    if file_patterns is None:
        file_patterns = ["*"]
    return _get_files(file_patterns, _glob_pjoin(HERE, root))


def _compile_pattern(pat, ignore_case=True):
    """Translate and compile a glob pattern to a regular expression matcher."""
    if isinstance(pat, bytes):
        pat_str = pat.decode("ISO-8859-1")
        res_str = _translate_glob(pat_str)
        res = res_str.encode("ISO-8859-1")
    else:
        res = _translate_glob(pat)
    flags = re.IGNORECASE if ignore_case else 0
    return re.compile(res, flags=flags).match


def _iexplode_path(path):
    """Iterate over all the parts of a path.

    Splits path recursively with os.path.split().
    """
    (head, tail) = os.path.split(path)
    if not head or (not tail and head == path):
        if head:
            yield head
        if tail or not head:
            yield tail
        return
    for p in _iexplode_path(head):
        yield p
    yield tail


def _translate_glob(pat):
    """Translate a glob PATTERN to a regular expression."""
    translated_parts = []
    for part in _iexplode_path(pat):
        translated_parts.append(_translate_glob_part(part))
    os_sep_class = "[%s]" % re.escape(SEPARATORS)
    res = _join_translated(translated_parts, os_sep_class)
    return "{res}\\Z(?ms)".format(res=res)


def _join_translated(translated_parts, os_sep_class):
    """Join translated glob pattern parts.

    This is different from a simple join, as care need to be taken
    to allow ** to match ZERO or more directories.
    """
    res = ""
    for part in translated_parts[:-1]:
        if part == ".*":
            # drop separator, since it is optional
            # (** matches ZERO or more dirs)
            res += part
        else:
            res += part + os_sep_class

    if translated_parts[-1] == ".*":
        # Final part is **
        res += ".+"
        # Follow stdlib/git convention of matching all sub files/directories:
        res += "({os_sep_class}?.*)?".format(os_sep_class=os_sep_class)
    else:
        res += translated_parts[-1]
    return res


def _translate_glob_part(pat):
    """Translate a glob PATTERN PART to a regular expression."""
    # Code modified from Python 3 standard lib fnmatch:
    if pat == "**":
        return ".*"
    i, n = 0, len(pat)
    res = []
    while i < n:
        c = pat[i]
        i = i + 1
        if c == "*":
            # Match anything but path separators:
            res.append("[^%s]*" % SEPARATORS)
        elif c == "?":
            res.append("[^%s]?" % SEPARATORS)
        elif c == "[":
            j = i
            if j < n and pat[j] == "!":
                j = j + 1
            if j < n and pat[j] == "]":
                j = j + 1
            while j < n and pat[j] != "]":
                j = j + 1
            if j >= n:
                res.append("\\[")
            else:
                stuff = pat[i:j].replace("\\", "\\\\")
                i = j + 1
                if stuff[0] == "!":
                    stuff = "^" + stuff[1:]
                elif stuff[0] == "^":
                    stuff = "\\" + stuff
                res.append("[%s]" % stuff)
        else:
            res.append(re.escape(c))
    return "".join(res)



import json
import os
import sys

from .serverapp import ServerApp
from jupyterlab import labapp
from traitlets import List, Unicode, default, observe

from jupyter_server.utils import url_path_join

from ._version import __version__
from .server_extension import _load_jupyter_server_extension
from .voila_handlers import add_voila_handlers

from .commands import *

HERE = os.path.abspath(os.path.dirname(__file__))


_examples = """
phoila build                       # build bundle
phoila list                        # list all configured extensions
phoila install <extension name>    # install an extension
phoila uninstall <extension name>  # uninstall an extension
"""


class PhoilaApp(ServerApp):
    """Base jupyter labextension command entry point"""

    name = "phoila"
    version = __version__
    description = "The Phoila application"
    examples = _examples

    default_url = Unicode(
        "/phoila", config=True, help="The default URL to redirect to from `/`"
    )

    subcommands = dict(
        install=(InstallPhoilaExtensionApp, "Install phoila extension(s)"),
        update=(UpdatePhoilaExtensionApp, "Update phoila extension(s)"),
        uninstall=(UninstallPhoilaExtensionApp, "Uninstall phoila extension(s)"),
        list=(ListLPhoilaxtensionsApp, "List phoila extensions"),
        link=(LinkPhoilaExtensionApp, "Link phoila extension(s)"),
        unlink=(UnlinkPhoilaExtensionApp, "Unlink phoila extension(s)"),
        enable=(EnableLPhoilaxtensionsApp, "Enable phoila extension(s)"),
        disable=(DisableLPhoilaxtensionsApp, "Disable phoila extension(s)"),
        check=(CheckLPhoilaxtensionsApp, "Check phoila extension(s)"),
        build=(PhoilaBuildApp, labapp.LabBuildApp.description.splitlines()[0]),
        clean=(PhoilaCleanApp, labapp.LabCleanApp.description.splitlines()[0]),
    )

    default_services = List(
        Unicode(),
        config=False,  # Not user configurable!
        help="default services to load",
        default_value=(
            "jupyter_server.kernelspecs.handlers",
            "jupyter_server.services.api.handlers",
            "jupyter_server.services.config.handlers",
            "jupyter_server.services.kernels.handlers",
            "jupyter_server.services.kernelspecs.handlers",
            "jupyter_server.services.security.handlers",
            "jupyter_server.services.shutdown",
        ),
    )

    mathjax_url = Unicode(
        "",
        config=True,
        help="""A custom url for MathJax.js.
        Should be in the form of a case-sensitive url to MathJax,
        for example:  /static/components/MathJax/MathJax.js
        """,
    )

    @default("mathjax_url")
    def _default_mathjax_url(self):
        if not self.enable_mathjax:
            return u""
        return "https://cdnjs.cloudflare.com/ajax/libs/mathjax/2.7.6/MathJax.js"

    @observe("mathjax_url")
    def _update_mathjax_url(self, change):
        new = change["new"]
        if new and not self.enable_mathjax:
            # enable_mathjax=False overrides mathjax_url
            self.mathjax_url = u""
        else:
            self.log.info("Using MathJax: %s", new)

    mathjax_config = Unicode(
        "TeX-AMS-MML_HTMLorMML-full,Safe",
        config=True,
        help="""The MathJax.js configuration file that is to be used.""",
    )

    @observe("mathjax_config")
    def _update_mathjax_config(self, change):
        self.log.info("Using MathJax configuration file: %s", change["new"])

    def initialize(self, *args, **kwargs):
        """Load the extension we need.
        """
        super(PhoilaApp, self).initialize(*args, **kwargs)
        if self.subapp is None:
            _load_jupyter_server_extension(self)
            add_voila_handlers(self)


def main():
    os.environ["JUPYTERLAB_DIR"] = os.path.join(
        sys.prefix, "share", "jupyter", "phoila"
    )
    labextensions.BaseExtensionApp.app_dir = os.environ["JUPYTERLAB_DIR"]

    PhoilaApp.launch_instance()


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
# coding: utf-8

# Copyright (c) Vidar Tonaas Fauske.
# Distributed under the terms of the Modified BSD License.

from ._version import __version__, version_info


def load_jupyter_server_extension(nb_server_app):
    # Wrap this here to avoid pulling in webapp in a normal run
    from .server_extension import _load_jupyter_server_extension

    _load_jupyter_server_extension(nb_server_app)


def _jupyter_server_extension_paths():
    return [{"module": "phoila"}]



import os
import sys

from traitlets import Instance, Unicode, default, HasTraits
from jupyterlab import labextensions, labapp
from jupyterlab.coreconfig import CoreConfig

APP_DIR_DEFAULT = os.path.join(sys.prefix, "share", "jupyter", "phoila")

extensions = (
    "@jupyterlab/application-extension",
    "@jupyterlab/apputils-extension",
    "@jupyterlab/csvviewer-extension",
    "@jupyterlab/docmanager-extension",
    "@jupyterlab/htmlviewer-extension",
    "@jupyterlab/imageviewer-extension",
    "@jupyterlab/markdownviewer-extension",
    "@jupyterlab/mathjax2-extension",
    "@jupyterlab/rendermime-extension",
    "@jupyterlab/shortcuts-extension",
    "@jupyterlab/theme-dark-extension",
    "@jupyterlab/theme-light-extension",
    "@jupyterlab/vdom-extension",
)
mime_extensions = (
    "@jupyterlab/javascript-extension",
    "@jupyterlab/json-extension",
    "@jupyterlab/pdf-extension",
    "@jupyterlab/vega4-extension",
    "@jupyterlab/vega5-extension",
)
singletons = (
    "@jupyterlab/application",
    "@jupyterlab/apputils",
    "@jupyterlab/console",
    "@jupyterlab/coreutils",
    "@jupyterlab/docmanager",
    "@jupyterlab/extensionmanager",
    "@jupyterlab/filebrowser",
    "@jupyterlab/fileeditor",
    "@jupyterlab/imageviewer",
    "@jupyterlab/launcher",
    "@jupyterlab/notebook",
    "@jupyterlab/rendermime",
    "@jupyterlab/rendermime-interfaces",
    "@jupyterlab/services",
    "@jupyterlab/terminal",
    "@jupyterlab/tooltip",
)


class PhoilaMixin(HasTraits):
    app_dir = Unicode(APP_DIR_DEFAULT, config=True, help="The app directory to target")

    core_config = Instance(CoreConfig, allow_none=True)

    @default("core_config")
    def _default_core_config(self):
        c = CoreConfig()
        orig_extensions = c.extensions
        orig_mimes = c.mime_extensions
        orig_singletons = c.singletons
        c.clear_packages()
        for e in extensions:
            c.add(e, orig_extensions[e], extension=True)
        for e in mime_extensions:
            c.add(e, orig_mimes[e], mime_extension=True)
        for e in singletons:
            c.add(e, orig_singletons[e])
        return c


class InstallPhoilaExtensionApp(PhoilaMixin, labextensions.InstallLabExtensionApp):
    pass


class UpdatePhoilaExtensionApp(PhoilaMixin, labextensions.UpdateLabExtensionApp):
    pass


class UninstallPhoilaExtensionApp(PhoilaMixin, labextensions.UninstallLabExtensionApp):
    pass


class ListLPhoilaxtensionsApp(PhoilaMixin, labextensions.ListLabExtensionsApp):
    pass


class LinkPhoilaExtensionApp(PhoilaMixin, labextensions.LinkLabExtensionApp):
    pass


class UnlinkPhoilaExtensionApp(PhoilaMixin, labextensions.UnlinkLabExtensionApp):
    pass


class EnableLPhoilaxtensionsApp(PhoilaMixin, labextensions.EnableLabExtensionsApp):
    pass


class DisableLPhoilaxtensionsApp(PhoilaMixin, labextensions.DisableLabExtensionsApp):
    pass


class CheckLPhoilaxtensionsApp(PhoilaMixin, labextensions.CheckLabExtensionsApp):
    pass


class PhoilaBuildApp(PhoilaMixin, labapp.LabBuildApp):
    def start(self):
        # Ensure we override the `sys_dir` in lab when building:
        os.environ["JUPYTERLAB_DIR"] = self.app_dir
        super(PhoilaBuildApp, self).start()


class PhoilaCleanApp(PhoilaMixin, labapp.LabCleanApp):
    pass



"""
The functions in this file is based on code from JupyterLab,
copied under the following license:

Copyright (c) 2015 Project Jupyter Contributors
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its
   contributors may be used to endorse or promote products derived from
   this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Semver File License
===================

The semver.py file is from https://github.com/podhmo/python-semver
which is licensed under the "MIT" license.  See the semver.py file for details.
"""

import os
import os.path as osp


def pjoin(*args):
    """Join paths to create a real path.
    """
    return osp.realpath(osp.join(*args))


def get_user_settings_dir(default=None):
    """Get the configured JupyterLab user settings directory.
    """
    settings_dir = os.environ.get("JUPYTERLAB_SETTINGS_DIR")
    settings_dir = (
        settings_dir
        or default
        or pjoin(jupyter_config_path()[0], "lab", "user-settings")
    )
    return osp.realpath(settings_dir)


def get_workspaces_dir(default=None):
    """Get the configured JupyterLab workspaces directory.
    """
    workspaces_dir = os.environ.get("JUPYTERLAB_WORKSPACES_DIR")
    workspaces_dir = (
        workspaces_dir
        or default
        or pjoin(jupyter_config_path()[0], "lab", "workspaces")
    )
    return osp.realpath(workspaces_dir)


def get_app_dir(default=None):
    """Get the configured JupyterLab app directory.
    """
    # Default to the override environment variable.
    if os.environ.get("JUPYTERLAB_DIR"):
        return osp.realpath(os.environ["JUPYTERLAB_DIR"])

    # Use the default locations for data_files.
    app_dir = default or pjoin(sys.prefix, "share", "jupyter", "lab")

    # Check for a user level install.
    # Ensure that USER_BASE is defined
    if hasattr(site, "getuserbase"):
        site.getuserbase()
    userbase = getattr(site, "USER_BASE", None)
    if HERE.startswith(userbase) and not app_dir.startswith(userbase):
        app_dir = pjoin(userbase, "share", "jupyter", "lab")

    # Check for a system install in '/usr/local/share'.
    elif (
        sys.prefix.startswith("/usr")
        and not osp.exists(app_dir)
        and osp.exists("/usr/local/share/jupyter/lab")
    ):
        app_dir = "/usr/local/share/jupyter/lab"

    return osp.realpath(app_dir)



# Copyright (c) Vidar Tonaas Fauske.
# Distributed under the terms of the Modified BSD License.

from tornado import web

from jupyter_server.base.handlers import APIHandler, JupyterHandler

from jupyterlab_server.handlers import (
    LabHandler as BaseLabHandler,
    ThemesHandler as BaseThemesHandler,
)
from jupyterlab_server.server import FileFindHandler as BaseFileFindHandler
from jupyterlab_server.workspaces_handler import (
    WorkspacesHandler as BaseWorkspacesHandler,
)
from jupyterlab_server.settings_handler import SettingsHandler as BaseSettingsHandler


class LabHandler(BaseLabHandler, JupyterHandler):
    pass


class ThemesHandler(BaseThemesHandler, JupyterHandler):
    pass


class FileFindHandler(BaseFileFindHandler, JupyterHandler):
    pass


class WorkspacesHandler(BaseWorkspacesHandler, APIHandler):
    @web.authenticated
    def put(self, space_name=""):
        if space_name == "phoila-single-workspace":
            self.set_status(204)
        else:
            super(WorkspacesHandler, self).put(space_name=space_name)


class SettingsHandler(BaseSettingsHandler, APIHandler):
    pass



"""
This code is based on code from voila, copied under the following license:

BSD License

Copyright (c) 2018 Voila contributors.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

  a. Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.

  b. Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.

  c. Neither the name of the authors nor the names of the contributors to
     this package may be used to endorse or promote products
     derived from this software without specific prior written
     permission.


THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
"""

import os
import gettext

from jinja2 import Environment, FileSystemLoader

from jupyter_server.utils import url_path_join
from jupyter_server.base.handlers import path_regex
from jupyter_server.base.handlers import FileFindHandler

from voila.paths import ROOT, STATIC_ROOT, collect_template_paths, jupyter_path
from voila.handler import VoilaHandler
from voila.treehandler import VoilaTreeHandler
from voila.static_file_handler import MultiStaticFileHandler, WhiteListFileHandler
from voila.configuration import VoilaConfiguration


HERE = os.path.dirname(__file__)
# if the directory above us contains the following paths, it means we are installed in dev mode (pip install -e .)
DEV_MODE = os.path.exists(os.path.join(HERE, "../setup.py")) and os.path.exists(
    os.path.join(HERE, "../share")
)


class PhoilaHandler(VoilaHandler):
    def set_header(self, header, value):
        if header == "Content-Type" and value == "text/html":
            value = "application/json"
        super(PhoilaHandler, self).set_header(header, value)


def add_voila_handlers(server_app):
    web_app = server_app.web_app

    nbconvert_template_paths = []
    static_paths = [STATIC_ROOT]
    template_paths = []

    # common configuration options between the server extension and the application
    voila_configuration = VoilaConfiguration(parent=server_app)
    voila_configuration.template = "phoila"
    voila_configuration.enable_nbextensions = False

    if DEV_MODE:
        search_directories = [
            os.path.abspath(
                os.path.join(HERE, "..", "share", "jupyter", "voila", "templates")
            )
        ]
        collect_template_paths(
            nbconvert_template_paths,
            static_paths,
            template_paths,
            voila_configuration.template,
            search_directories,
        )
    else:
        collect_template_paths(
            nbconvert_template_paths,
            static_paths,
            template_paths,
            voila_configuration.template,
        )

    jenv_opt = {"autoescape": True}
    env = Environment(
        loader=FileSystemLoader(template_paths),
        extensions=["jinja2.ext.i18n"],
        **jenv_opt
    )
    web_app.settings["voila_jinja2_env"] = env

    nbui = gettext.translation(
        "nbui", localedir=os.path.join(ROOT, "i18n"), fallback=True
    )
    env.install_gettext_translations(nbui, newstyle=False)

    host_pattern = ".*$"
    base_url = url_path_join(web_app.settings["base_url"])

    web_app.add_handlers(
        host_pattern,
        [
            (
                url_path_join(base_url, r"/voila/render/(.*)"),
                PhoilaHandler,
                {
                    "config": server_app.config,
                    "nbconvert_template_paths": nbconvert_template_paths,
                    "voila_configuration": voila_configuration,
                },
            ),
            (url_path_join(base_url, "/voila"), VoilaTreeHandler),
            (url_path_join(base_url, r"/voila/tree" + path_regex), VoilaTreeHandler),
            (
                url_path_join(base_url, r"/voila/static/(.*)"),
                MultiStaticFileHandler,
                {"paths": static_paths},
            ),
            (url_path_join(base_url, r"/voila/files/(.*)"), WhiteListFileHandler),
        ],
    )

    # Serving notebook extensions
    if voila_configuration.enable_nbextensions:
        # First look into 'nbextensions_path' configuration key (classic notebook)
        # and fall back to default path for nbextensions (jupyter server).
        if "nbextensions_path" in web_app.settings:
            nbextensions_path = web_app.settings["nbextensions_path"]
        else:
            nbextensions_path = jupyter_path("nbextensions")

        web_app.add_handlers(
            host_pattern,
            [
                # this handler serves the nbextensions similar to the classical notebook
                (
                    url_path_join(base_url, r"/voila/nbextensions/(.*)"),
                    FileFindHandler,
                    {
                        "path": nbextensions_path,
                        "no_cache_paths": ["/"],  # don't cache anything in nbextensions
                    },
                )
            ],
        )



# Copyright (c) Vidar Tonaas Fauske.
# Distributed under the terms of the Modified BSD License.

import os
import sys

import tornado.web
from traitlets import HasTraits, Bool, Unicode, default
from jupyter_core.paths import jupyter_config_path
from jupyterlab.commands import get_app_info
from jupyterlab_server.handlers import is_url, LabConfig
from jupyterlab_server.server import url_path_join as ujoin

from .lab_handlers import (
    LabHandler,
    ThemesHandler,
    FileFindHandler,
    WorkspacesHandler,
    SettingsHandler,
)

from ._version import __version__
from .app_config import get_app_dir, get_user_settings_dir, get_workspaces_dir, pjoin
from .commands import APP_DIR_DEFAULT

HERE = os.path.dirname(__file__)

USER_SETTINGS_DIR_DEFAULT = pjoin(jupyter_config_path()[0], "phoila", "user-settings")
WORKSPACES_DIR_DEFAULT = pjoin(jupyter_config_path()[0], "phoila", "workspaces")


def load_config(nbapp):
    config = LabConfig(app_url="/phoila", tree_url="/voila/tree")
    app_dir = getattr(nbapp, "app_dir", get_app_dir(default=APP_DIR_DEFAULT))

    info = get_app_info(app_dir)
    static_url = info["staticUrl"]
    user_settings_dir = getattr(
        nbapp,
        "user_settings_dir",
        get_user_settings_dir(default=USER_SETTINGS_DIR_DEFAULT),
    )
    workspaces_dir = getattr(
        nbapp, "workspaces_dir", get_workspaces_dir(default=WORKSPACES_DIR_DEFAULT)
    )

    config.app_dir = app_dir
    config.app_name = "Phoila"
    config.app_namespace = "phoila"
    config.app_settings_dir = os.path.join(app_dir, "settings")
    config.cache_files = True
    config.schemas_dir = os.path.join(app_dir, "schemas")
    config.templates_dir = os.path.join(app_dir, "static")
    config.themes_dir = os.path.join(app_dir, "themes")
    config.user_settings_dir = user_settings_dir
    config.workspaces_dir = workspaces_dir

    if getattr(nbapp, "override_static_url", ""):
        static_url = nbapp.override_static_url
    if getattr(nbapp, "override_theme_url", ""):
        config.themes_url = nbapp.override_theme_url
        config.themes_dir = ""

    if static_url:
        config.static_url = static_url
    else:
        config.static_dir = os.path.join(app_dir, "static")

    return config


def _load_jupyter_server_extension(jupyter_app):
    if "JUPYTERLAB_DIR" not in os.environ:
        os.environ["JUPYTERLAB_DIR"] = os.path.join(
            sys.prefix, "share", "jupyter", "phoila"
        )
    if "JUPYTERLAB_SETTINGS_DIR" not in os.environ:
        os.environ["JUPYTERLAB_SETTINGS_DIR"] = os.path.join(
            jupyter_config_path()[0], "phoila", "settings"
        )
    if "JUPYTERLAB_WORKSPACES_DIR" not in os.environ:
        os.environ["JUPYTERLAB_WORKSPACES_DIR"] = os.path.join(
            jupyter_config_path()[0], "phoila", "workspaces"
        )

    settings = jupyter_app.web_app.settings
    if "mathjax_url" not in settings:
        settings["mathjax_url"] = jupyter_app.mathjax_url
    if "mathjax_config" not in settings:
        settings["mathjax_config"] = jupyter_app.mathjax_config

    config = load_config(jupyter_app)
    return add_handlers(jupyter_app.web_app, config)


def add_handlers(web_app, config):
    """Add the appropriate handlers to the web app.
    """
    # Normalize directories.
    for name in config.trait_names():
        if not name.endswith("_dir"):
            continue
        value = getattr(config, name)
        setattr(config, name, value.replace(os.sep, "/"))

    # Normalize urls
    # Local urls should have a leading slash but no trailing slash
    for name in config.trait_names():
        if not name.endswith("_url"):
            continue
        value = getattr(config, name)
        if is_url(value):
            continue
        if not value.startswith("/"):
            value = "/" + value
        if value.endswith("/"):
            value = value[:-1]
        setattr(config, name, value)

    # Set up the main page handler and tree handler.
    base_url = web_app.settings.get("base_url", "/")
    app_path = ujoin(base_url, config.app_url)
    handlers = [
        (app_path, LabHandler, {"lab_config": config}),
        #  + notebook_path_regex + '?'
    ]

    # Cache all or none of the files depending on the `cache_files` setting.
    no_cache_paths = [] if config.cache_files else ["/"]

    # Handle single notebook mode:
    single_mode_path = ujoin(app_path, "single", r".+")
    handlers.append((single_mode_path, LabHandler, {"lab_config": config}))
    print(single_mode_path)

    # Handle local static assets.
    if config.static_dir:
        static_path = ujoin(base_url, config.static_url, "(.*)")
        handlers.append(
            (
                static_path,
                FileFindHandler,
                {"path": config.static_dir, "no_cache_paths": no_cache_paths},
            )
        )

    # Handle local settings.
    if config.schemas_dir:
        settings_config = {
            "app_settings_dir": config.app_settings_dir,
            "schemas_dir": config.schemas_dir,
            "settings_dir": config.user_settings_dir,
        }

        # Handle requests for the list of settings. Make slash optional.
        settings_path = ujoin(base_url, config.settings_url, "?")
        handlers.append((settings_path, SettingsHandler, settings_config))

        # Handle requests for an individual set of settings.
        setting_path = ujoin(base_url, config.settings_url, "(?P<schema_name>.+)")
        handlers.append((setting_path, SettingsHandler, settings_config))

    # Handle saved workspaces.
    if config.workspaces_dir:
        # Handle JupyterLab client URLs that include workspaces.
        workspaces_path = ujoin(base_url, config.workspaces_url, r".+")
        handlers.append((workspaces_path, LabHandler, {"lab_config": config}))

        workspaces_config = {
            "workspaces_url": config.workspaces_url,
            "path": config.workspaces_dir,
        }

        # Handle requests for the list of workspaces. Make slash optional.
        workspaces_api_path = ujoin(base_url, config.workspaces_api_url, "?")
        handlers.append((workspaces_api_path, WorkspacesHandler, workspaces_config))

        # Handle requests for an individually named workspace.
        workspace_api_path = ujoin(
            base_url, config.workspaces_api_url, "(?P<space_name>.+)"
        )
        handlers.append((workspace_api_path, WorkspacesHandler, workspaces_config))

    # Handle local themes.
    if config.themes_dir:
        themes_url = ujoin(base_url, config.themes_url)
        themes_path = ujoin(themes_url, "(.*)")
        handlers.append(
            (
                themes_path,
                ThemesHandler,
                {
                    "themes_url": themes_url,
                    "path": config.themes_dir,
                    "no_cache_paths": no_cache_paths,
                },
            )
        )

    web_app.add_handlers(".*$", handlers)



#!/usr/bin/env python
# coding: utf-8

# Copyright (c) Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.

from __future__ import print_function
from glob import glob
from os.path import join as pjoin


from setupbase import (
    create_cmdclass,
    install_npm,
    ensure_targets,
    find_packages,
    combine_commands,
    ensure_python,
    get_version,
    HERE,
)

from setuptools import setup


# The name of the project
name = "phoila"

# Ensure a valid python version
ensure_python(">=3.5")

# Get our version
version = get_version(pjoin(name, "_version.py"))

js_path = pjoin(HERE, "ts")
lab_path = pjoin(HERE, "ts", "dist")

# Representative files that should exist after a successful build
jstargets = [pjoin(js_path, "lib", "plugins.js")]

package_data_spec = {name: ["staging/*.*"]}

data_files_spec = [
    ("", HERE, "etc/**/*"),
    ("share/jupyter/phoila/extensions", lab_path, "*.tgz"),
    ("", HERE, "share/**/*"),
]


cmdclass = create_cmdclass(
    "jsdeps", package_data_spec=package_data_spec, data_files_spec=data_files_spec
)
cmdclass["jsdeps"] = combine_commands(
    install_npm(js_path, build_cmd="build"), ensure_targets(jstargets)
)


setup_args = dict(
    name=name,
    description="A Phosphor wrapper for Voila",
    version=version,
    scripts=glob(pjoin("scripts", "*")),
    cmdclass=cmdclass,
    packages=find_packages(),
    author="Vidar Tonaas Fauske",
    author_email="vidartf@gmail.com",
    url="https://github.com/vidartf/phoila",
    license="BSD",
    platforms="Linux, Mac OS X, Windows",
    keywords=["Jupyter", "Widgets", "IPython"],
    classifiers=[
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Framework :: Jupyter",
    ],
    include_package_data=True,
    install_requires=[
        "ipywidgets>=7.0.0",
        "jupyter_server>=0.1.1",
        "jupyterlab>=1.1.0",
        "voila>=0.1.10",
    ],
    extras_require={"test": ["pytest>=3.6", "pytest-cov"]},
    entry_points={"console_scripts": ["phoila = phoila.app:main"]},
)

if __name__ == "__main__":
    setup(**setup_args)
